@@ -1,5 +1,5 @@
 /* Internationalization Tag Set (ITS) handling
-   Copyright (C) 2015-2024 Free Software Foundation, Inc.
+   Copyright (C) 2015-2025 Free Software Foundation, Inc.
 
    This file was written by Daiki Ueno <ueno@gnu.org>, 2015.
 
@@ -28,6 +28,7 @@
 #include <stdint.h>
 #include <stdlib.h>
 
+#include <libxml/xmlversion.h>
 #include <libxml/xmlerror.h>
 #include <libxml/tree.h>
 #include <libxml/parser.h>
@@ -35,6 +36,7 @@
 #include <libxml/xpath.h>
 #include <libxml/xpathInternals.h>
 
+#define SB_NO_APPENDF
 #include <error.h>
 #include "mem-hash-map.h"
 #include "trim.h"
@@ -82,7 +84,13 @@
 /* ----------------------------- Error handling ----------------------------- */
 
 static void
+/* Adapt to API change in libxml 2.12.0.
+   See <https://gitlab.gnome.org/GNOME/libxml2/-/issues/622>.  */
+#if LIBXML_VERSION >= 21200
+structured_error (void *data, const xmlError *err)
+#else
 structured_error (void *data, xmlError *err)
+#endif
 {
   error (0, err->level == XML_ERR_FATAL ? EXIT_FAILURE : 0,
          _("%s error: %s"), "libxml2", err->message);
@@ -2235,9 +2243,9 @@ set_doc_encoding_utf8 (xmlDoc *doc)
       doc->encoding = BAD_CAST xstrdup ("UTF-8");
       return true;
     }
-  string_desc_t enc = string_desc_from_c ((char *) doc->encoding);
-  if (string_desc_c_casecmp (enc, string_desc_from_c ("UTF-8")) == 0
-      || string_desc_c_casecmp (enc, string_desc_from_c ("UTF8")) == 0)
+  string_desc_t enc = sd_from_c ((char *) doc->encoding);
+  if (sd_c_casecmp (enc, sd_from_c ("UTF-8")) == 0
+      || sd_c_casecmp (enc, sd_from_c ("UTF8")) == 0)
     return true;
   /* The document's encoding is not UTF-8.  Conversion would be expensive.  */
   return false;
@@ -2333,9 +2341,8 @@ _its_is_valid_simple_gen_xml (const char *contents,
           if (add_to_node != NULL && !slash_before_tag)
             {
               string_desc_t name =
-                string_desc_new_addr (name_end - name_start,
-                                      (char *) name_start);
-              char *name_c = xstring_desc_c (name);
+                sd_new_addr (name_end - name_start, name_start);
+              char *name_c = xsd_c (name);
               if (ignore_case)
                 {
                   /* Convert the name to lower case.  */
@@ -2426,13 +2433,13 @@ _its_is_valid_simple_gen_xml (const char *contents,
                   if (add_to_node != NULL)
                     {
                       string_desc_t attr_name =
-                        string_desc_new_addr (attr_name_end - attr_name_start,
-                                              (char *) attr_name_start);
+                        sd_new_addr (attr_name_end - attr_name_start,
+                                     attr_name_start);
                       string_desc_t attr_value =
-                        string_desc_new_addr (attr_value_end - attr_value_start,
-                                              (char *) attr_value_start);
-                      char *attr_name_c = xstring_desc_c (attr_name);
-                      char *attr_value_c = xstring_desc_c (attr_value);
+                        sd_new_addr (attr_value_end - attr_value_start,
+                                     attr_value_start);
+                      char *attr_name_c = xsd_c (attr_name);
+                      char *attr_value_c = xsd_c (attr_value);
                       xmlAttr *attr =
                         xmlNewProp (current_node, BAD_CAST attr_name_c,
                                     BAD_CAST attr_value_c);
@@ -2467,8 +2474,7 @@ _its_is_valid_simple_gen_xml (const char *contents,
             return false;
           /* Seen a complete <...> element start/end.  */
           /* Verify that the tag is allowed.  */
-          string_desc_t tag =
-            string_desc_new_addr (name_end - name_start, (char *) name_start);
+          string_desc_t tag = sd_new_addr (name_end - name_start, name_start);
           if (!(valid_element == NULL || valid_element (tag)))
             return false;
           if (slash_after_tag || (no_end_element != NULL && no_end_element (tag)))
@@ -2492,8 +2498,9 @@ _its_is_valid_simple_gen_xml (const char *contents,
               if (open_elements_count == 0)
                 /* The end of an element without a corresponding start.  */
                 return false;
-              if ((ignore_case ? string_desc_c_casecmp : string_desc_cmp)
-                  (open_elements[open_elements_count - 1], tag)
+              if ((ignore_case
+                   ? sd_c_casecmp (open_elements[open_elements_count - 1], tag)
+                   : sd_cmp (open_elements[open_elements_count - 1], tag))
                   != 0)
                 return false;
               open_elements_count--;
@@ -2659,7 +2666,7 @@ is_valid_xhtml_element (string_desc_t tag)
       /* Invariant:
          If tag occurs in the table, it is at an index >= lo, < hi.  */
       size_t i = (lo + hi) / 2; /* >= lo, < hi */
-      int cmp = string_desc_cmp (tag, string_desc_from_c (allowed[i]));
+      int cmp = sd_cmp (tag, sd_from_c (allowed[i]));
       if (cmp == 0)
         return true;
       if (cmp < 0)
@@ -2757,7 +2764,7 @@ is_valid_html_element (string_desc_t tag)
       /* Invariant:
          If tag occurs in the table, it is at an index >= lo, < hi.  */
       size_t i = (lo + hi) / 2; /* >= lo, < hi */
-      int cmp = string_desc_cmp (tag, string_desc_from_c (allowed[i]));
+      int cmp = sd_cmp (tag, sd_from_c (allowed[i]));
       if (cmp == 0)
         return true;
       if (cmp < 0)
@@ -2774,8 +2781,8 @@ is_no_end_html_element (string_desc_t tag)
   /* Specification:
      https://html.spec.whatwg.org/
      Search for "Tag omission in text/html: No end tag."  */
-  return string_desc_cmp (tag, string_desc_from_c ("br")) == 0
-         || string_desc_cmp (tag, string_desc_from_c ("hr")) == 0;
+  return sd_cmp (tag, sd_from_c ("br")) == 0
+         || sd_cmp (tag, sd_from_c ("hr")) == 0;
 }
 
 /* Returns true if the argument is a piece of simple well-formed HTML
